package com.qmth.themis.admin.api;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TBAttachment;
import com.qmth.themis.business.entity.TBUser;
import com.qmth.themis.business.entity.TEAudio;
import com.qmth.themis.business.entity.TEExamActivity;
import com.qmth.themis.business.service.TBAttachmentService;
import com.qmth.themis.business.service.TEAudioService;
import com.qmth.themis.business.service.TEExamActivityService;
import com.qmth.themis.business.service.TEExamPaperService;
import com.qmth.themis.business.util.OssUtil;
import com.qmth.themis.business.util.ServletUtil;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.io.File;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * @Description: 考试场次 前端控制器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Api(tags = "考试场次Controller")
@RestController
@RequestMapping("/${prefix.url.admin}/activity")
@Validated
public class TEExamActivityController {

    @Resource
    private TEExamActivityService teExamActivityService;

    @Resource
    private TEExamPaperService examPaperService;

    @Resource
    TEAudioService teAudioService;

    @Resource
    OssUtil ossUtil;

    @Resource
    TBAttachmentService tbAttachmentService;

    @ApiOperation(value = "考试场次修改/新增接口")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    public Result save(
            @ApiParam(value = "考试场次信息", required = true) @RequestBody List<TEExamActivity> teExamActivityList) {
        teExamActivityService.saveExamActivity(teExamActivityList);
        examPaperService.disposeObjectiveAnswer(teExamActivityList.get(0).getExamId());
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "考试场次查询接口")
    @RequestMapping(value = "/query", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "考试场次信息", response = TEExamActivity.class)})
    public Result query(@ApiParam(value = "主键", required = false) @RequestParam(required = false) Long id,
                        @ApiParam(value = "考试批次id", required = false) @RequestParam(required = false) Long examId,
                        @ApiParam(value = "考试场次编码", required = false) @RequestParam(required = false) String code,
                        @ApiParam(value = "开始日期", required = false) @RequestParam(required = false) Long startDate,
                        @ApiParam(value = "结束日期", required = false) @RequestParam(required = false) Long finishDate,
                        @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) int pageNumber,
                        @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.SELECT_PAGE_SIZE_MAX) int pageSize) {
        return ResultUtil.ok(teExamActivityService
                .examActivityQuery(new Page<>(pageNumber, pageSize), id, examId, code, startDate, finishDate));
    }

    @ApiOperation(value = "考试场次语音查询接口")
    @RequestMapping(value = "/audio/query", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "考试场次语音信息", response = TEAudio.class)})
    public Result audioQuery(@ApiParam(value = "考试场次id", required = true) @RequestParam Long activityId,
                             @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) int pageNumber,
                             @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) int pageSize) {
        IPage<TEAudio> teAudioIPage = teAudioService.query(new Page<>(pageNumber, pageSize), activityId);
        if (Objects.nonNull(teAudioIPage) && !CollectionUtils.isEmpty(teAudioIPage.getRecords())) {
            for (TEAudio t : teAudioIPage.getRecords()) {
                if (Objects.nonNull(t.getAttachmentId())) {
                    TBAttachment tbAttachment = tbAttachmentService.getById(t.getAttachmentId());
                    if (Objects.nonNull(tbAttachment.getRemark())) {
                        JSONObject jsonObject = JSONObject.parseObject(tbAttachment.getRemark());
                        t.setAttachmentPath(ossUtil.getAliYunOssPublicDomain().getPublicUrl() + File.separator + jsonObject.getString(SystemConstant.PATH));
                    }
                }
            }
        }
        return ResultUtil.ok(teAudioIPage);
    }

    @ApiOperation(value = "考试场次语音保存接口")
    @RequestMapping(value = "/audio/save", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "常规信息", response = Result.class)})
    @Transactional
    public Result audioSave(@Validated @RequestBody TEAudio teAudio, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        TBUser tbUser = (TBUser) ServletUtil.getRequestAccount();
        if (Objects.isNull(teAudio.getId())) {
            teAudio.insertInfo(tbUser.getId());
        } else {
            teAudio.updateInfo(tbUser.getId());
        }
        teAudioService.saveOrUpdate(teAudio);
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "考试场次语音启用/禁用接口")
    @RequestMapping(value = "/audio/enable", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "常规信息", response = Result.class)})
    @Transactional
    public Result audioEnable(@ApiParam(value = "考试场次语音id", required = true) @RequestParam Long audioId,
                              @ApiParam(value = "启用/禁用", required = true) @RequestParam Boolean enable) {
        TBUser tbUser = (TBUser) ServletUtil.getRequestAccount();
        TEAudio teAudio = teAudioService.getById(audioId);
        Optional.ofNullable(teAudio).orElseThrow(() -> new BusinessException("语音信息不存在"));
        teAudio.setEnable(enable);
        teAudio.updateInfo(tbUser.getId());
        teAudioService.updateById(teAudio);
        return ResultUtil.ok(true);
    }
}
